@@ -14,13 +14,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector.ipc;
 
 import static org.apache.arrow.vector.TestUtils.newVarCharVector;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertArrayEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.IOException;
 import java.math.BigDecimal;
@@ -32,7 +34,6 @@ import java.time.ZoneId;
 import java.time.ZoneOffset;
 import java.util.Arrays;
 import java.util.List;
-
 import org.apache.arrow.memory.ArrowBuf;
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.memory.RootAllocator;
@@ -84,37 +85,40 @@ import org.apache.arrow.vector.types.pojo.Field;
 import org.apache.arrow.vector.types.pojo.FieldType;
 import org.apache.arrow.vector.util.JsonStringArrayList;
 import org.apache.arrow.vector.util.Text;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-/**
- * Helps testing the file formats.
- */
+/** Helps testing the file formats. */
 public class BaseFileTest {
   private static final Logger LOGGER = LoggerFactory.getLogger(BaseFileTest.class);
   protected static final int COUNT = 10;
   protected BufferAllocator allocator;
 
-  @Before
+  @BeforeEach
   public void init() {
     allocator = new RootAllocator(Integer.MAX_VALUE);
   }
 
-  @After
+  @AfterEach
   public void tearDown() {
     allocator.close();
   }
 
-
-  private static short [] uint1Values = new short[]{0, 255, 1, 128, 2};
-  private static char [] uint2Values = new char[]{0, Character.MAX_VALUE, 1, Short.MAX_VALUE * 2, 2};
-  private static long [] uint4Values = new long[]{0, Integer.MAX_VALUE + 1L, 1, Integer.MAX_VALUE * 2L, 2};
-  private static BigInteger[] uint8Values = new BigInteger[]{BigInteger.valueOf(0),
-      BigInteger.valueOf(Long.MAX_VALUE).multiply(BigInteger.valueOf(2)), BigInteger.valueOf(2),
-      BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.valueOf(1)), BigInteger.valueOf(2)};
+  private static short[] uint1Values = new short[] {0, 255, 1, 128, 2};
+  private static char[] uint2Values =
+      new char[] {0, Character.MAX_VALUE, 1, Short.MAX_VALUE * 2, 2};
+  private static long[] uint4Values =
+      new long[] {0, Integer.MAX_VALUE + 1L, 1, Integer.MAX_VALUE * 2L, 2};
+  private static BigInteger[] uint8Values =
+      new BigInteger[] {
+        BigInteger.valueOf(0),
+        BigInteger.valueOf(Long.MAX_VALUE).multiply(BigInteger.valueOf(2)),
+        BigInteger.valueOf(2),
+        BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.valueOf(1)),
+        BigInteger.valueOf(2)
+      };
 
   protected void writeData(int count, StructVector parent) {
     ComplexWriter writer = new ComplexWriterImpl("root", parent);
@@ -132,11 +136,11 @@ public class BaseFileTest {
       uint1Writer.setPosition(i);
       // TODO: Fix add safe write methods on uint methods.
       uint1Writer.setPosition(i);
-      uint1Writer.writeUInt1((byte) uint1Values[i % uint1Values.length] );
+      uint1Writer.writeUInt1((byte) uint1Values[i % uint1Values.length]);
       uint2Writer.setPosition(i);
-      uint2Writer.writeUInt2((char) uint2Values[i % uint2Values.length] );
+      uint2Writer.writeUInt2((char) uint2Values[i % uint2Values.length]);
       uint4Writer.setPosition(i);
-      uint4Writer.writeUInt4((int) uint4Values[i % uint4Values.length] );
+      uint4Writer.writeUInt4((int) uint4Values[i % uint4Values.length]);
       uint8Writer.setPosition(i);
       uint8Writer.writeUInt8(uint8Values[i % uint8Values.length].longValue());
       bigIntWriter.setPosition(i);
@@ -147,20 +151,26 @@ public class BaseFileTest {
     writer.setValueCount(count);
   }
 
-
   protected void validateContent(int count, VectorSchemaRoot root) {
     for (int i = 0; i < count; i++) {
-      Assert.assertEquals(i, root.getVector("int").getObject(i));
-      Assert.assertEquals((Short) uint1Values[i % uint1Values.length],
+      assertEquals(i, root.getVector("int").getObject(i));
+      assertEquals(
+          (Short) uint1Values[i % uint1Values.length],
           ((UInt1Vector) root.getVector("uint1")).getObjectNoOverflow(i));
-      Assert.assertEquals("Failed for index: " + i, (Character) uint2Values[i % uint2Values.length],
-          (Character) ((UInt2Vector) root.getVector("uint2")).get(i));
-      Assert.assertEquals("Failed for index: " + i, (Long) uint4Values[i % uint4Values.length],
-          ((UInt4Vector) root.getVector("uint4")).getObjectNoOverflow(i));
-      Assert.assertEquals("Failed for index: " + i, uint8Values[i % uint8Values.length],
-          ((UInt8Vector) root.getVector("uint8")).getObjectNoOverflow(i));
-      Assert.assertEquals(Long.valueOf(i), root.getVector("bigInt").getObject(i));
-      Assert.assertEquals(i == 0 ? Float.NaN : i, root.getVector("float").getObject(i));
+      assertEquals(
+          (Character) uint2Values[i % uint2Values.length],
+          (Character) ((UInt2Vector) root.getVector("uint2")).get(i),
+          "Failed for index: " + i);
+      assertEquals(
+          (Long) uint4Values[i % uint4Values.length],
+          ((UInt4Vector) root.getVector("uint4")).getObjectNoOverflow(i),
+          "Failed for index: " + i);
+      assertEquals(
+          uint8Values[i % uint8Values.length],
+          ((UInt8Vector) root.getVector("uint8")).getObjectNoOverflow(i),
+          "Failed for index: " + i);
+      assertEquals(Long.valueOf(i), root.getVector("bigInt").getObject(i));
+      assertEquals(i == 0 ? Float.NaN : i, root.getVector("float").getObject(i));
     }
   }
 
@@ -210,23 +220,23 @@ public class BaseFileTest {
   }
 
   protected void validateComplexContent(int count, VectorSchemaRoot root) {
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
     printVectors(root.getFieldVectors());
     for (int i = 0; i < count; i++) {
 
       Object intVal = root.getVector("int").getObject(i);
       if (i % 5 != 3) {
-        Assert.assertEquals(i, intVal);
+        assertEquals(i, intVal);
       } else {
-        Assert.assertNull(intVal);
+        assertNull(intVal);
       }
-      Assert.assertEquals(Long.valueOf(i), root.getVector("bigInt").getObject(i));
-      Assert.assertEquals(i % 3, ((List<?>) root.getVector("list").getObject(i)).size());
+      assertEquals(Long.valueOf(i), root.getVector("bigInt").getObject(i));
+      assertEquals(i % 3, ((List<?>) root.getVector("list").getObject(i)).size());
       NullableTimeStampMilliHolder h = new NullableTimeStampMilliHolder();
       FieldReader structReader = root.getVector("struct").getReader();
       structReader.setPosition(i);
       structReader.reader("timestamp").read(h);
-      Assert.assertEquals(i, h.value);
+      assertEquals(i, h.value);
     }
   }
 
@@ -235,61 +245,67 @@ public class BaseFileTest {
   }
 
   protected void writeDateTimeData(int count, StructVector parent) {
-    Assert.assertTrue(count < 100);
+    assertTrue(count < 100);
     ComplexWriter writer = new ComplexWriterImpl("root", parent);
     StructWriter rootWriter = writer.rootAsStruct();
     DateMilliWriter dateWriter = rootWriter.dateMilli("date");
     TimeMilliWriter timeWriter = rootWriter.timeMilli("time");
     TimeStampMilliWriter timeStampMilliWriter = rootWriter.timeStampMilli("timestamp-milli");
-    TimeStampMilliTZWriter timeStampMilliTZWriter = rootWriter.timeStampMilliTZ("timestamp-milliTZ", "Europe/Paris");
+    TimeStampMilliTZWriter timeStampMilliTZWriter =
+        rootWriter.timeStampMilliTZ("timestamp-milliTZ", "Europe/Paris");
     TimeStampNanoWriter timeStampNanoWriter = rootWriter.timeStampNano("timestamp-nano");
     for (int i = 0; i < count; i++) {
       LocalDateTime dt = makeDateTimeFromCount(i);
-      // Number of days in milliseconds since epoch, stored as 64-bit integer, only date part is used
+      // Number of days in milliseconds since epoch, stored as 64-bit integer, only date part is
+      // used
       dateWriter.setPosition(i);
       long dateLong = dt.toLocalDate().atStartOfDay().toInstant(ZoneOffset.UTC).toEpochMilli();
       dateWriter.writeDateMilli(dateLong);
       // Time is a value in milliseconds since midnight, stored as 32-bit integer
       timeWriter.setPosition(i);
-      int milliOfDay = (int) java.util.concurrent.TimeUnit.NANOSECONDS.toMillis(dt.toLocalTime().toNanoOfDay());
+      int milliOfDay =
+          (int) java.util.concurrent.TimeUnit.NANOSECONDS.toMillis(dt.toLocalTime().toNanoOfDay());
       timeWriter.writeTimeMilli(milliOfDay);
       // Timestamp as milliseconds since the epoch, stored as 64-bit integer
       timeStampMilliWriter.setPosition(i);
       timeStampMilliWriter.writeTimeStampMilli(dt.toInstant(ZoneOffset.UTC).toEpochMilli());
       // Timestamp as milliseconds since epoch with timezone
       timeStampMilliTZWriter.setPosition(i);
-      timeStampMilliTZWriter.writeTimeStampMilliTZ(dt.atZone(ZoneId.of("Europe/Paris")).toInstant().toEpochMilli());
+      timeStampMilliTZWriter.writeTimeStampMilliTZ(
+          dt.atZone(ZoneId.of("Europe/Paris")).toInstant().toEpochMilli());
       // Timestamp as nanoseconds since epoch
       timeStampNanoWriter.setPosition(i);
-      long tsNanos = dt.toInstant(ZoneOffset.UTC).toEpochMilli() * 1_000_000 + i; // need to add back in nano val
+      long tsNanos =
+          dt.toInstant(ZoneOffset.UTC).toEpochMilli() * 1_000_000
+              + i; // need to add back in nano val
       timeStampNanoWriter.writeTimeStampNano(tsNanos);
     }
     writer.setValueCount(count);
   }
 
   protected void validateDateTimeContent(int count, VectorSchemaRoot root) {
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
     printVectors(root.getFieldVectors());
     for (int i = 0; i < count; i++) {
       LocalDateTime dt = makeDateTimeFromCount(i);
       LocalDateTime dtMilli = dt.minusNanos(i);
       LocalDateTime dateVal = ((DateMilliVector) root.getVector("date")).getObject(i);
       LocalDateTime dateExpected = dt.toLocalDate().atStartOfDay();
-      Assert.assertEquals(dateExpected, dateVal);
+      assertEquals(dateExpected, dateVal);
       LocalTime timeVal = ((TimeMilliVector) root.getVector("time")).getObject(i).toLocalTime();
-      Assert.assertEquals(dtMilli.toLocalTime(), timeVal);
+      assertEquals(dtMilli.toLocalTime(), timeVal);
       Object timestampMilliVal = root.getVector("timestamp-milli").getObject(i);
-      Assert.assertEquals(dtMilli, timestampMilliVal);
+      assertEquals(dtMilli, timestampMilliVal);
       Object timestampMilliTZVal = root.getVector("timestamp-milliTZ").getObject(i);
-      Assert.assertEquals(dt.atZone(ZoneId.of("Europe/Paris")).toInstant().toEpochMilli(), timestampMilliTZVal);
+      assertEquals(
+          dt.atZone(ZoneId.of("Europe/Paris")).toInstant().toEpochMilli(), timestampMilliTZVal);
       Object timestampNanoVal = root.getVector("timestamp-nano").getObject(i);
-      Assert.assertEquals(dt, timestampNanoVal);
+      assertEquals(dt, timestampNanoVal);
     }
   }
 
   protected VectorSchemaRoot writeFlatDictionaryData(
-      BufferAllocator bufferAllocator,
-      DictionaryProvider.MapDictionaryProvider provider) {
+      BufferAllocator bufferAllocator, DictionaryProvider.MapDictionaryProvider provider) {
 
     // Define dictionaries and add to provider
     VarCharVector dictionary1Vector = newVarCharVector("D1", bufferAllocator);
@@ -299,7 +315,8 @@ public class BaseFileTest {
     dictionary1Vector.set(2, "baz".getBytes(StandardCharsets.UTF_8));
     dictionary1Vector.setValueCount(3);
 
-    Dictionary dictionary1 = new Dictionary(dictionary1Vector, new DictionaryEncoding(1L, false, null));
+    Dictionary dictionary1 =
+        new Dictionary(dictionary1Vector, new DictionaryEncoding(1L, false, null));
     provider.put(dictionary1);
 
     VarCharVector dictionary2Vector = newVarCharVector("D2", bufferAllocator);
@@ -309,7 +326,8 @@ public class BaseFileTest {
     dictionary2Vector.set(2, "large".getBytes(StandardCharsets.UTF_8));
     dictionary2Vector.setValueCount(3);
 
-    Dictionary dictionary2 = new Dictionary(dictionary2Vector, new DictionaryEncoding(2L, false, null));
+    Dictionary dictionary2 =
+        new Dictionary(dictionary2Vector, new DictionaryEncoding(2L, false, null));
     provider.put(dictionary2);
 
     // Populate the vectors
@@ -346,80 +364,81 @@ public class BaseFileTest {
     FieldVector encodedVector2 = (FieldVector) DictionaryEncoder.encode(vector2, dictionary2);
     vector2.close(); // Done with this vector after encoding
 
-    List<Field> fields = Arrays.asList(encodedVector1A.getField(), encodedVector1B.getField(),
-        encodedVector2.getField());
-    List<FieldVector> vectors = Collections2.asImmutableList(encodedVector1A, encodedVector1B, encodedVector2);
+    List<Field> fields =
+        Arrays.asList(
+            encodedVector1A.getField(), encodedVector1B.getField(), encodedVector2.getField());
+    List<FieldVector> vectors =
+        Collections2.asImmutableList(encodedVector1A, encodedVector1B, encodedVector2);
 
     return new VectorSchemaRoot(fields, vectors, encodedVector1A.getValueCount());
   }
 
   protected void validateFlatDictionary(VectorSchemaRoot root, DictionaryProvider provider) {
     FieldVector vector1A = root.getVector("varcharA");
-    Assert.assertNotNull(vector1A);
+    assertNotNull(vector1A);
 
     DictionaryEncoding encoding1A = vector1A.getField().getDictionary();
-    Assert.assertNotNull(encoding1A);
-    Assert.assertEquals(1L, encoding1A.getId());
+    assertNotNull(encoding1A);
+    assertEquals(1L, encoding1A.getId());
 
-    Assert.assertEquals(6, vector1A.getValueCount());
-    Assert.assertEquals(0, vector1A.getObject(0));
-    Assert.assertEquals(1, vector1A.getObject(1));
-    Assert.assertEquals(null, vector1A.getObject(2));
-    Assert.assertEquals(2, vector1A.getObject(3));
-    Assert.assertEquals(1, vector1A.getObject(4));
-    Assert.assertEquals(2, vector1A.getObject(5));
+    assertEquals(6, vector1A.getValueCount());
+    assertEquals(0, vector1A.getObject(0));
+    assertEquals(1, vector1A.getObject(1));
+    assertEquals(null, vector1A.getObject(2));
+    assertEquals(2, vector1A.getObject(3));
+    assertEquals(1, vector1A.getObject(4));
+    assertEquals(2, vector1A.getObject(5));
 
     FieldVector vector1B = root.getVector("varcharB");
-    Assert.assertNotNull(vector1B);
+    assertNotNull(vector1B);
 
     DictionaryEncoding encoding1B = vector1A.getField().getDictionary();
-    Assert.assertNotNull(encoding1B);
-    Assert.assertTrue(encoding1A.equals(encoding1B));
-    Assert.assertEquals(1L, encoding1B.getId());
+    assertNotNull(encoding1B);
+    assertTrue(encoding1A.equals(encoding1B));
+    assertEquals(1L, encoding1B.getId());
 
-    Assert.assertEquals(6, vector1B.getValueCount());
-    Assert.assertEquals(2, vector1B.getObject(0));
-    Assert.assertEquals(1, vector1B.getObject(1));
-    Assert.assertEquals(2, vector1B.getObject(2));
-    Assert.assertEquals(null, vector1B.getObject(3));
-    Assert.assertEquals(1, vector1B.getObject(4));
-    Assert.assertEquals(0, vector1B.getObject(5));
+    assertEquals(6, vector1B.getValueCount());
+    assertEquals(2, vector1B.getObject(0));
+    assertEquals(1, vector1B.getObject(1));
+    assertEquals(2, vector1B.getObject(2));
+    assertEquals(null, vector1B.getObject(3));
+    assertEquals(1, vector1B.getObject(4));
+    assertEquals(0, vector1B.getObject(5));
 
     FieldVector vector2 = root.getVector("sizes");
-    Assert.assertNotNull(vector2);
+    assertNotNull(vector2);
 
     DictionaryEncoding encoding2 = vector2.getField().getDictionary();
-    Assert.assertNotNull(encoding2);
-    Assert.assertEquals(2L, encoding2.getId());
+    assertNotNull(encoding2);
+    assertEquals(2L, encoding2.getId());
 
-    Assert.assertEquals(6, vector2.getValueCount());
-    Assert.assertEquals(null, vector2.getObject(0));
-    Assert.assertEquals(2, vector2.getObject(1));
-    Assert.assertEquals(1, vector2.getObject(2));
-    Assert.assertEquals(1, vector2.getObject(3));
-    Assert.assertEquals(2, vector2.getObject(4));
-    Assert.assertEquals(null, vector2.getObject(5));
+    assertEquals(6, vector2.getValueCount());
+    assertEquals(null, vector2.getObject(0));
+    assertEquals(2, vector2.getObject(1));
+    assertEquals(1, vector2.getObject(2));
+    assertEquals(1, vector2.getObject(3));
+    assertEquals(2, vector2.getObject(4));
+    assertEquals(null, vector2.getObject(5));
 
     Dictionary dictionary1 = provider.lookup(1L);
-    Assert.assertNotNull(dictionary1);
+    assertNotNull(dictionary1);
     VarCharVector dictionaryVector = ((VarCharVector) dictionary1.getVector());
-    Assert.assertEquals(3, dictionaryVector.getValueCount());
-    Assert.assertEquals(new Text("foo"), dictionaryVector.getObject(0));
-    Assert.assertEquals(new Text("bar"), dictionaryVector.getObject(1));
-    Assert.assertEquals(new Text("baz"), dictionaryVector.getObject(2));
+    assertEquals(3, dictionaryVector.getValueCount());
+    assertEquals(new Text("foo"), dictionaryVector.getObject(0));
+    assertEquals(new Text("bar"), dictionaryVector.getObject(1));
+    assertEquals(new Text("baz"), dictionaryVector.getObject(2));
 
     Dictionary dictionary2 = provider.lookup(2L);
-    Assert.assertNotNull(dictionary2);
+    assertNotNull(dictionary2);
     dictionaryVector = ((VarCharVector) dictionary2.getVector());
-    Assert.assertEquals(3, dictionaryVector.getValueCount());
-    Assert.assertEquals(new Text("micro"), dictionaryVector.getObject(0));
-    Assert.assertEquals(new Text("small"), dictionaryVector.getObject(1));
-    Assert.assertEquals(new Text("large"), dictionaryVector.getObject(2));
+    assertEquals(3, dictionaryVector.getValueCount());
+    assertEquals(new Text("micro"), dictionaryVector.getObject(0));
+    assertEquals(new Text("small"), dictionaryVector.getObject(1));
+    assertEquals(new Text("large"), dictionaryVector.getObject(2));
   }
 
   protected VectorSchemaRoot writeNestedDictionaryData(
-      BufferAllocator bufferAllocator,
-      DictionaryProvider.MapDictionaryProvider provider) {
+      BufferAllocator bufferAllocator, DictionaryProvider.MapDictionaryProvider provider) {
 
     // Define the dictionary and add to the provider
     VarCharVector dictionaryVector = newVarCharVector("D2", bufferAllocator);
@@ -428,7 +447,8 @@ public class BaseFileTest {
     dictionaryVector.set(1, "bar".getBytes(StandardCharsets.UTF_8));
     dictionaryVector.setValueCount(2);
 
-    Dictionary dictionary = new Dictionary(dictionaryVector, new DictionaryEncoding(2L, false, null));
+    Dictionary dictionary =
+        new Dictionary(dictionaryVector, new DictionaryEncoding(2L, false, null));
     provider.put(dictionary);
 
     // Write the vector data using dictionary indices
@@ -456,26 +476,26 @@ public class BaseFileTest {
 
   protected void validateNestedDictionary(VectorSchemaRoot root, DictionaryProvider provider) {
     FieldVector vector = root.getFieldVectors().get(0);
-    Assert.assertNotNull(vector);
-    Assert.assertNull(vector.getField().getDictionary());
+    assertNotNull(vector);
+    assertNull(vector.getField().getDictionary());
     Field nestedField = vector.getField().getChildren().get(0);
 
     DictionaryEncoding encoding = nestedField.getDictionary();
-    Assert.assertNotNull(encoding);
-    Assert.assertEquals(2L, encoding.getId());
-    Assert.assertEquals(new ArrowType.Int(32, true), encoding.getIndexType());
+    assertNotNull(encoding);
+    assertEquals(2L, encoding.getId());
+    assertEquals(new ArrowType.Int(32, true), encoding.getIndexType());
 
-    Assert.assertEquals(3, vector.getValueCount());
-    Assert.assertEquals(Arrays.asList(0, 1), vector.getObject(0));
-    Assert.assertEquals(Arrays.asList(0), vector.getObject(1));
-    Assert.assertEquals(Arrays.asList(1), vector.getObject(2));
+    assertEquals(3, vector.getValueCount());
+    assertEquals(Arrays.asList(0, 1), vector.getObject(0));
+    assertEquals(Arrays.asList(0), vector.getObject(1));
+    assertEquals(Arrays.asList(1), vector.getObject(2));
 
     Dictionary dictionary = provider.lookup(2L);
-    Assert.assertNotNull(dictionary);
+    assertNotNull(dictionary);
     VarCharVector dictionaryVector = ((VarCharVector) dictionary.getVector());
-    Assert.assertEquals(2, dictionaryVector.getValueCount());
-    Assert.assertEquals(new Text("foo"), dictionaryVector.getObject(0));
-    Assert.assertEquals(new Text("bar"), dictionaryVector.getObject(1));
+    assertEquals(2, dictionaryVector.getValueCount());
+    assertEquals(new Text("foo"), dictionaryVector.getObject(0));
+    assertEquals(new Text("bar"), dictionaryVector.getObject(1));
   }
 
   protected VectorSchemaRoot writeDecimalData(BufferAllocator bufferAllocator) {
@@ -498,9 +518,11 @@ public class BaseFileTest {
     decimalVector2.setValueCount(count);
     decimalVector3.setValueCount(count);
 
-    List<Field> fields = Collections2.asImmutableList(decimalVector1.getField(), decimalVector2.getField(),
-        decimalVector3.getField());
-    List<FieldVector> vectors = Collections2.asImmutableList(decimalVector1, decimalVector2, decimalVector3);
+    List<Field> fields =
+        Collections2.asImmutableList(
+            decimalVector1.getField(), decimalVector2.getField(), decimalVector3.getField());
+    List<FieldVector> vectors =
+        Collections2.asImmutableList(decimalVector1, decimalVector2, decimalVector3);
     return new VectorSchemaRoot(fields, vectors, count);
   }
 
@@ -509,26 +531,26 @@ public class BaseFileTest {
     DecimalVector decimalVector2 = (DecimalVector) root.getVector("decimal2");
     DecimalVector decimalVector3 = (DecimalVector) root.getVector("decimal3");
     int count = 10;
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
 
     for (int i = 0; i < count; i++) {
       // Verify decimal 1 vector
       BigDecimal readValue = decimalVector1.getObject(i);
       ArrowType.Decimal type = (ArrowType.Decimal) decimalVector1.getField().getType();
       BigDecimal genValue = new BigDecimal(BigInteger.valueOf(i), type.getScale());
-      Assert.assertEquals(genValue, readValue);
+      assertEquals(genValue, readValue);
 
       // Verify decimal 2 vector
       readValue = decimalVector2.getObject(i);
       type = (ArrowType.Decimal) decimalVector2.getField().getType();
       genValue = new BigDecimal(BigInteger.valueOf(i * (1 << 10)), type.getScale());
-      Assert.assertEquals(genValue, readValue);
+      assertEquals(genValue, readValue);
 
       // Verify decimal 3 vector
       readValue = decimalVector3.getObject(i);
       type = (ArrowType.Decimal) decimalVector3.getField().getType();
       genValue = new BigDecimal(BigInteger.valueOf(i * 1111111111111111L), type.getScale());
-      Assert.assertEquals(genValue, readValue);
+      assertEquals(genValue, readValue);
     }
   }
 
@@ -538,7 +560,8 @@ public class BaseFileTest {
     nullVector1.setValueCount(valueCount);
     nullVector2.setValueCount(valueCount);
 
-    List<Field> fields = Collections2.asImmutableList(nullVector1.getField(), nullVector2.getField());
+    List<Field> fields =
+        Collections2.asImmutableList(nullVector1.getField(), nullVector2.getField());
     List<FieldVector> vectors = Collections2.asImmutableList(nullVector1, nullVector2);
     return new VectorSchemaRoot(fields, vectors, valueCount);
   }
@@ -556,20 +579,23 @@ public class BaseFileTest {
     FieldReader unionReader = root.getVector("union").getReader();
     for (int i = 0; i < count; i++) {
       unionReader.setPosition(i);
-      switch (i % 4) {
+      switch (i % 5) {
         case 0:
-          Assert.assertEquals(i, unionReader.readInteger().intValue());
+          assertEquals(i, unionReader.readInteger().intValue());
           break;
         case 1:
-          Assert.assertEquals(i, unionReader.readLong().longValue());
+          assertEquals(i, unionReader.readLong().longValue());
           break;
         case 2:
-          Assert.assertEquals(i % 3, unionReader.size());
+          assertEquals(i % 3, unionReader.size());
           break;
         case 3:
+          assertEquals(3, unionReader.size());
+          break;
+        case 4:
           NullableTimeStampMilliHolder h = new NullableTimeStampMilliHolder();
           unionReader.reader("timestamp").read(h);
-          Assert.assertEquals(i, h.value);
+          assertEquals(i, h.value);
           break;
         default:
           assert false : "Unexpected value in switch statement: " + i;
@@ -589,9 +615,10 @@ public class BaseFileTest {
     IntWriter intWriter = rootWriter.integer("union");
     BigIntWriter bigIntWriter = rootWriter.bigInt("union");
     ListWriter listWriter = rootWriter.list("union");
+    ListWriter listViewWriter = rootWriter.listView("union");
     StructWriter structWriter = rootWriter.struct("union");
     for (int i = 0; i < count; i++) {
-      switch (i % 4) {
+      switch (i % 5) {
         case 0:
           intWriter.setPosition(i);
           intWriter.writeInt(i);
@@ -609,6 +636,14 @@ public class BaseFileTest {
           listWriter.endList();
           break;
         case 3:
+          listViewWriter.setPosition(i);
+          listViewWriter.startListView();
+          for (int j = 0; j < i % 5; j++) {
+            listViewWriter.varChar().writeVarChar(0, 3, varchar);
+          }
+          listViewWriter.endListView();
+          break;
+        case 4:
           structWriter.setPosition(i);
           structWriter.start();
           structWriter.timeStampMilli("timestamp").writeTimeStampMilli(i);
@@ -623,7 +658,7 @@ public class BaseFileTest {
   }
 
   protected void writeVarBinaryData(int count, StructVector parent) {
-    Assert.assertTrue(count < 100);
+    assertTrue(count < 100);
     ComplexWriter writer = new ComplexWriterImpl("root", parent);
     StructWriter rootWriter = writer.rootAsStruct();
     ListWriter listWriter = rootWriter.list("list");
@@ -642,7 +677,7 @@ public class BaseFileTest {
   }
 
   protected void validateVarBinary(int count, VectorSchemaRoot root) {
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
     ListVector listVector = (ListVector) root.getVector("list");
     byte[] expectedArray = new byte[count];
     int numVarBinaryValues = 0;
@@ -650,26 +685,27 @@ public class BaseFileTest {
       expectedArray[i] = (byte) i;
       List<?> objList = listVector.getObject(i);
       if (i % 3 == 0) {
-        Assert.assertTrue(objList.isEmpty());
+        assertTrue(objList.isEmpty());
       } else {
         byte[] expected = Arrays.copyOfRange(expectedArray, 0, i + 1);
         for (int j = 0; j < i % 3; j++) {
           byte[] result = (byte[]) objList.get(j);
-          Assert.assertArrayEquals(result, expected);
+          assertArrayEquals(result, expected);
           numVarBinaryValues++;
         }
       }
     }
 
     // ListVector lastSet should be the index of last value + 1
-    Assert.assertEquals(listVector.getLastSet(), count - 1);
+    assertEquals(listVector.getLastSet(), count - 1);
 
     // VarBinaryVector lastSet should be the index of last value
     VarBinaryVector binaryVector = (VarBinaryVector) listVector.getChildrenFromFields().get(0);
-    Assert.assertEquals(binaryVector.getLastSet(), numVarBinaryValues - 1);
+    assertEquals(binaryVector.getLastSet(), numVarBinaryValues - 1);
   }
 
-  protected void writeBatchData(ArrowWriter writer, IntVector vector, VectorSchemaRoot root) throws IOException {
+  protected void writeBatchData(ArrowWriter writer, IntVector vector, VectorSchemaRoot root)
+      throws IOException {
     writer.start();
 
     vector.setNull(0);
@@ -752,7 +788,8 @@ public class BaseFileTest {
     mapWriter.setValueCount(COUNT);
     sortedMapWriter.setValueCount(COUNT);
 
-    List<Field> fields = Collections2.asImmutableList(mapVector.getField(), sortedMapVector.getField());
+    List<Field> fields =
+        Collections2.asImmutableList(mapVector.getField(), sortedMapVector.getField());
     List<FieldVector> vectors = Collections2.asImmutableList(mapVector, sortedMapVector);
     return new VectorSchemaRoot(fields, vectors, count);
   }
@@ -762,7 +799,7 @@ public class BaseFileTest {
     MapVector sortedMapVector = (MapVector) root.getVector("mapSorted");
 
     final int count = 10;
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
 
     UnionMapReader mapReader = new UnionMapReader(mapVector);
     UnionMapReader sortedMapReader = new UnionMapReader(sortedMapVector);
@@ -801,9 +838,12 @@ public class BaseFileTest {
     ListVector mapEntryList = ListVector.empty("entryList", bufferAllocator);
     FieldType mapEntryType = new FieldType(false, ArrowType.Struct.INSTANCE, null, null);
     StructVector mapEntryData = new StructVector("entryData", bufferAllocator, mapEntryType, null);
-    mapEntryData.addOrGet("myKey", new FieldType(false, new ArrowType.Int(64, true), null), BigIntVector.class);
-    mapEntryData.addOrGet("myValue", FieldType.nullable(new ArrowType.Int(32, true)), IntVector.class);
-    mapEntryList.initializeChildrenFromFields(Collections2.asImmutableList(mapEntryData.getField()));
+    mapEntryData.addOrGet(
+        "myKey", new FieldType(false, new ArrowType.Int(64, true), null), BigIntVector.class);
+    mapEntryData.addOrGet(
+        "myValue", FieldType.nullable(new ArrowType.Int(32, true)), IntVector.class);
+    mapEntryList.initializeChildrenFromFields(
+        Collections2.asImmutableList(mapEntryData.getField()));
     UnionListWriter entryWriter = mapEntryList.getWriter();
     entryWriter.allocate();
 
@@ -833,7 +873,7 @@ public class BaseFileTest {
     MapVector sortedMapVector = (MapVector) root.getVector("map");
 
     final int count = 10;
-    Assert.assertEquals(count, root.getRowCount());
+    assertEquals(count, root.getRowCount());
 
     UnionMapReader sortedMapReader = new UnionMapReader(sortedMapVector);
     sortedMapReader.setKeyValueNames("myKey", "myValue");
