@@ -74,6 +74,17 @@ test_that("transmute", {
   )
 })
 
+test_that("transmute after group_by", {
+  compare_dplyr_binding(
+    .input %>%
+      select(int, dbl, chr) %>%
+      group_by(chr, int) %>%
+      transmute(dbl + 1) %>%
+      collect(),
+    tbl
+  )
+})
+
 test_that("transmute respect bespoke dplyr implementation", {
   ## see: https://github.com/tidyverse/dplyr/issues/6086
   compare_dplyr_binding(
@@ -141,16 +152,14 @@ test_that("transmute() with unsupported arguments", {
 })
 
 test_that("transmute() defuses dots arguments (ARROW-13262)", {
-  expect_warning(
+  expect_snapshot(
     tbl %>%
       Table$create() %>%
       transmute(
         a = stringr::str_c(padded_strings, padded_strings),
         b = stringr::str_squish(a)
       ) %>%
-      collect(),
-    "Expression stringr::str_squish(a) not supported in Arrow; pulling data into R",
-    fixed = TRUE
+      collect()
   )
 })
 
@@ -191,10 +200,7 @@ test_that("nchar() arguments", {
       filter(line_lengths > 15) %>%
       collect(),
     tbl,
-    warning = paste0(
-      "In nchar\\(verses, type = \"bytes\", allowNA = TRUE\\), ",
-      "allowNA = TRUE not supported in Arrow; pulling data into R"
-    )
+    warning = "allowNA = TRUE not supported in Arrow"
   )
 })
 
@@ -367,18 +373,16 @@ test_that("dplyr::mutate's examples", {
   # The mutate operation may yield different results on grouped
   # tibbles because the expressions are computed within groups.
   # The following normalises `mass` by the global average:
-  # TODO(ARROW-13926): support window functions
   compare_dplyr_binding(
     .input %>%
       select(name, mass, species) %>%
       mutate(mass_norm = mass / mean(mass, na.rm = TRUE)) %>%
       collect(),
-    starwars,
-    warning = "window function"
+    starwars
   )
 })
 
-test_that("Can mutate after group_by as long as there are no aggregations", {
+test_that("Can mutate after group_by, including with some aggregations", {
   compare_dplyr_binding(
     .input %>%
       select(int, chr) %>%
@@ -397,27 +401,108 @@ test_that("Can mutate after group_by as long as there are no aggregations", {
       collect(),
     tbl
   )
-  expect_warning(
-    tbl %>%
-      Table$create() %>%
+  # Check the column order when .keep = "none"
+  compare_dplyr_binding(
+    .input %>%
+      select(chr, int) %>%
+      group_by(chr) %>%
+      mutate(int + 1, .keep = "none") %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
       select(int, chr) %>%
       group_by(chr) %>%
       mutate(avg_int = mean(int)) %>%
+      # Because this silently does a join, the rows can get unsorted
+      arrange(chr) %>%
       collect(),
-    "window functions not currently supported in Arrow; pulling data into R",
-    fixed = TRUE
+    tbl
   )
-  expect_warning(
-    tbl %>%
-      Table$create() %>%
+  compare_dplyr_binding(
+    .input %>%
       select(mean = int, chr) %>%
       # rename `int` to `mean` and use `mean(mean)` in `mutate()` to test that
       # `all_funs()` detects `mean()` despite the collision with a column name
       group_by(chr) %>%
       mutate(avg_int = mean(mean)) %>%
+      # Because this silently does a join, the rows can get unsorted
+      arrange(chr) %>%
       collect(),
-    "window functions not currently supported in Arrow; pulling data into R",
-    fixed = TRUE
+    tbl
+  )
+})
+
+test_that("Can mutate with .by argument, even with some aggregations", {
+  compare_dplyr_binding(
+    .input %>%
+      select(int, chr) %>%
+      mutate(int = int + 6L, .by = chr) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(int, chr) %>%
+      mutate(int = int + 6L, .by = starts_with("chr")) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(int, chr) %>%
+      mutate(new_col = int + 6L, .by = c(chr, int)) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(int, chr) %>%
+      mutate(new_col = int + 6L, .by = c("chr", "int")) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(mean = int, chr) %>%
+      # rename `int` to `mean` and use `mean` in `mutate()` to test that
+      # `all_funs()` does not incorrectly identify it as an aggregate function
+      mutate(mean = mean + 6L, .by = chr) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(int, chr) %>%
+      mutate(avg_int = mean(int), .by = chr) %>%
+      # Because this silently does a join, the rows can get unsorted
+      arrange(chr) %>%
+      collect(),
+    tbl
+  )
+  compare_dplyr_binding(
+    .input %>%
+      select(mean = int, chr) %>%
+      # rename `int` to `mean` and use `mean(mean)` in `mutate()` to test that
+      # `all_funs()` detects `mean()` despite the collision with a column name
+      mutate(avg_int = mean(mean), .by = chr) %>%
+      # Because this silently does a join, the rows can get unsorted
+      arrange(chr) %>%
+      collect(),
+    tbl
+  )
+})
+
+test_that("Can't supply .by after group_by", {
+  expect_error(
+    tbl %>%
+      arrow_table() %>%
+      select(int, chr) %>%
+      group_by(chr) %>%
+      mutate(int = int + 6L, .by = chr) %>%
+      collect(),
+    "Can't supply `\\.by` when `\\.data` is grouped data"
   )
 })
 
@@ -448,7 +533,7 @@ test_that("Can't just add a vector column with mutate()", {
         mutate(again = 1:10),
       tibble::tibble(int = tbl$int, again = 1:10)
     ),
-    "In again = 1:10, only values of size one are recycled; pulling data into R"
+    "Recycling values of length != 1 not supported in Arrow"
   )
 })
 
@@ -590,7 +675,6 @@ test_that("mutate() and transmute() with namespaced functions", {
 })
 
 test_that("Can use across() within mutate()", {
-
   # expressions work in the right order
   compare_dplyr_binding(
     .input %>%
@@ -625,17 +709,15 @@ test_that("Can use across() within mutate()", {
     example_data
   )
 
-  # gives the right error with window functions
-  expect_warning(
-    arrow_table(example_data) %>%
+  compare_dplyr_binding(
+    .input %>%
       mutate(
         x = int + 2,
         across(c("int", "dbl"), list(mean = mean, sd = sd, round)),
         exp(dbl2)
       ) %>%
       collect(),
-    "window functions not currently supported in Arrow; pulling data into R",
-    fixed = TRUE
+    example_data
   )
 })
 
@@ -652,3 +734,41 @@ test_that("Can use across() within transmute()", {
     example_data
   )
 })
+
+test_that("across() does not select grouping variables within mutate()", {
+  compare_dplyr_binding(
+    .input %>%
+      select(int, dbl, chr) %>%
+      group_by(chr) %>%
+      mutate(across(everything(), round)) %>%
+      collect(),
+    example_data
+  )
+
+  expect_error(
+    example_data %>%
+      arrow_table() %>%
+      group_by(chr) %>%
+      mutate(across(chr, as.character)),
+    "Column `chr` doesn't exist"
+  )
+})
+
+test_that("across() does not select grouping variables within transmute()", {
+  compare_dplyr_binding(
+    .input %>%
+      select(int, dbl, chr) %>%
+      group_by(chr) %>%
+      transmute(across(everything(), round)) %>%
+      collect(),
+    example_data
+  )
+
+  expect_error(
+    example_data %>%
+      arrow_table() %>%
+      group_by(chr) %>%
+      transmute(across(chr, as.character)),
+    "Column `chr` doesn't exist"
+  )
+})
