@@ -22,17 +22,18 @@
 #include <memory>
 #include <mutex>
 #include <numeric>
+#include <set>
 #include <sstream>
 
+#include "arrow/acero/exec_plan.h"
+#include "arrow/acero/options.h"
+#include "arrow/acero/query_context.h"
 #include "arrow/array/array_primitive.h"
 #include "arrow/array/util.h"
 #include "arrow/compute/api_aggregate.h"
 #include "arrow/compute/api_scalar.h"
 #include "arrow/compute/api_vector.h"
 #include "arrow/compute/cast.h"
-#include "arrow/compute/exec/exec_plan.h"
-#include "arrow/compute/exec/options.h"
-#include "arrow/compute/exec/query_context.h"
 #include "arrow/dataset/dataset.h"
 #include "arrow/dataset/dataset_internal.h"
 #include "arrow/dataset/plan.h"
@@ -135,28 +136,34 @@ Result<std::shared_ptr<Schema>> GetProjectedSchemaFromExpression(
     const std::shared_ptr<Schema>& dataset_schema) {
   // process resultant dataset_schema after projection
   FieldVector project_fields;
+  std::set<std::string> field_names;
   if (auto call = projection.call()) {
     if (call->function_name != "make_struct") {
       return Status::Invalid("Top level projection expression call must be make_struct");
     }
-    for (const compute::Expression& arg : call->arguments) {
-      if (auto field_ref = arg.field_ref()) {
-        if (field_ref->IsName()) {
-          auto field = dataset_schema->GetFieldByName(*field_ref->name());
-          if (field) {
-            project_fields.push_back(std::move(field));
-          }
-          // if the field is not present in the schema we ignore it.
-          // the case is if kAugmentedFields are present in the expression
-          // and if they are not present in the provided schema, we ignore them.
-        } else {
-          return Status::Invalid(
-              "No projected schema was supplied and we could not infer the projected "
-              "schema from the projection expression.");
-        }
+    for (auto field_ref : compute::FieldsInExpression(projection)) {
+      if (field_ref.IsName()) {
+        field_names.emplace(*field_ref.name());
+      } else if (field_ref.IsNested()) {
+        // We keep the top-level field name.
+        auto nested_field_refs = *field_ref.nested_refs();
+        field_names.emplace(*nested_field_refs[0].name());
+      } else {
+        return Status::Invalid(
+            "No projected schema was supplied and we could not infer the projected "
+            "schema from the projection expression.");
       }
     }
   }
+  for (auto f : field_names) {
+    auto field = dataset_schema->GetFieldByName(f);
+    if (field) {
+      // if the field is not present in the schema we ignore it.
+      // the case is if kAugmentedFields are present in the expression
+      // and if they are not present in the provided schema, we ignore them.
+      project_fields.push_back(std::move(field));
+    }
+  }
   return schema(project_fields);
 }
 
@@ -204,7 +211,8 @@ Status NormalizeScanOptions(const std::shared_ptr<ScanOptions>& scan_options,
         // create the projected schema only if the provided expressions
         // produces valid set of fields.
         ARROW_ASSIGN_OR_RAISE(auto projection_descr,
-                              ProjectionDescr::Default(*projected_schema));
+                              ProjectionDescr::Default(
+                                  *projected_schema, scan_options->add_augmented_fields));
         scan_options->projected_schema = std::move(projection_descr.schema);
         scan_options->projection = projection_descr.expression;
         ARROW_ASSIGN_OR_RAISE(scan_options->projection,
@@ -213,7 +221,8 @@ Status NormalizeScanOptions(const std::shared_ptr<ScanOptions>& scan_options,
         // if projected_fields are not found, we default to creating the projected_schema
         // and projection from the dataset_schema.
         ARROW_ASSIGN_OR_RAISE(auto projection_descr,
-                              ProjectionDescr::Default(*dataset_schema));
+                              ProjectionDescr::Default(
+                                  *dataset_schema, scan_options->add_augmented_fields));
         scan_options->projected_schema = std::move(projection_descr.schema);
         scan_options->projection = projection_descr.expression;
       }
@@ -224,7 +233,7 @@ Status NormalizeScanOptions(const std::shared_ptr<ScanOptions>& scan_options,
     ARROW_ASSIGN_OR_RAISE(
         auto projection_descr,
         ProjectionDescr::FromNames(scan_options->projected_schema->field_names(),
-                                   *dataset_schema));
+                                   *dataset_schema, scan_options->add_augmented_fields));
     scan_options->projection = projection_descr.expression;
   }
 
@@ -286,16 +295,14 @@ Result<EnumeratedRecordBatchGenerator> FragmentToBatches(
     const Enumerated<std::shared_ptr<Fragment>>& fragment,
     const std::shared_ptr<ScanOptions>& options) {
 #ifdef ARROW_WITH_OPENTELEMETRY
-  auto tracer = arrow::internal::tracing::GetTracer();
-  auto span = tracer->StartSpan(
-      "arrow::dataset::FragmentToBatches",
-      {
-          {"arrow.dataset.fragment", fragment.value->ToString()},
-          {"arrow.dataset.fragment.index", fragment.index},
-          {"arrow.dataset.fragment.last", fragment.last},
-          {"arrow.dataset.fragment.type_name", fragment.value->type_name()},
-      });
-  auto scope = tracer->WithActiveSpan(span);
+  util::tracing::Span span;
+  START_SPAN(span, "Scanner::FragmentToBatches",
+             {
+                 {"arrow.dataset.fragment", fragment.value->ToString()},
+                 {"arrow.dataset.fragment.index", fragment.index},
+                 {"arrow.dataset.fragment.last", fragment.last},
+                 {"arrow.dataset.fragment.type_name", fragment.value->type_name()},
+             });
 #endif
   ARROW_ASSIGN_OR_RAISE(auto batch_gen, fragment.value->ScanBatchesAsync(options));
   ArrayVector columns;
@@ -426,11 +433,11 @@ Result<EnumeratedRecordBatchGenerator> AsyncScanner::ScanBatchesUnorderedAsync(
   auto exec_context =
       std::make_shared<compute::ExecContext>(scan_options_->pool, cpu_executor);
 
-  compute::QueryOptions query_options;
+  acero::QueryOptions query_options;
   query_options.use_legacy_batching = use_legacy_batching;
 
   ARROW_ASSIGN_OR_RAISE(auto plan,
-                        compute::ExecPlan::Make(query_options, *exec_context.get()));
+                        acero::ExecPlan::Make(query_options, *exec_context.get()));
   AsyncGenerator<std::optional<compute::ExecBatch>> sink_gen;
 
   auto exprs = scan_options_->projection.call()->arguments;
@@ -439,18 +446,18 @@ Result<EnumeratedRecordBatchGenerator> AsyncScanner::ScanBatchesUnorderedAsync(
                    ->field_names;
 
   RETURN_NOT_OK(
-      compute::Declaration::Sequence(
+      acero::Declaration::Sequence(
           {
               {"scan", ScanNodeOptions{dataset_, scan_options_, sequence_fragments}},
-              {"filter", compute::FilterNodeOptions{scan_options_->filter}},
+              {"filter", acero::FilterNodeOptions{scan_options_->filter}},
               {"augmented_project",
-               compute::ProjectNodeOptions{std::move(exprs), std::move(names)}},
-              {"sink", compute::SinkNodeOptions{&sink_gen, /*schema=*/nullptr,
-                                                scan_options_->backpressure}},
+               acero::ProjectNodeOptions{std::move(exprs), std::move(names)}},
+              {"sink", acero::SinkNodeOptions{&sink_gen, /*schema=*/nullptr,
+                                              scan_options_->backpressure}},
           })
           .AddToPlan(plan.get()));
 
-  RETURN_NOT_OK(plan->StartProducing());
+  plan->StartProducing();
 
   auto options = scan_options_;
   ARROW_ASSIGN_OR_RAISE(auto fragments_it, dataset_->GetFragments(scan_options_->filter));
@@ -468,13 +475,24 @@ Result<EnumeratedRecordBatchGenerator> AsyncScanner::ScanBatchesUnorderedAsync(
         }
       }};
 
-  return MakeMappedGenerator(
+  EnumeratedRecordBatchGenerator mapped_gen = MakeMappedGenerator(
       std::move(sink_gen),
-      [sink_gen, options, stop_producing,
+      [sink_gen, options,
        shared_fragments](const std::optional<compute::ExecBatch>& batch)
           -> Future<EnumeratedRecordBatch> {
         return ToEnumeratedRecordBatch(batch, *options, *shared_fragments);
       });
+
+  return [mapped_gen = std::move(mapped_gen), plan = std::move(plan),
+          stop_producing = std::move(stop_producing)] {
+    auto next = mapped_gen();
+    return next.Then([plan](const EnumeratedRecordBatch& value) {
+      if (IsIterationEnd(value)) {
+        return plan->finished().Then([value] { return value; });
+      }
+      return Future<EnumeratedRecordBatch>::MakeFinished(value);
+    });
+  };
 }
 
 Result<std::shared_ptr<Table>> AsyncScanner::TakeRows(const Array& indices) {
@@ -709,12 +727,13 @@ Future<int64_t> AsyncScanner::CountRowsAsync(Executor* executor) {
 
   compute::ExecContext exec_context(scan_options_->pool, executor);
 
-  ARROW_ASSIGN_OR_RAISE(auto plan, compute::ExecPlan::Make(exec_context));
+  ARROW_ASSIGN_OR_RAISE(auto plan, acero::ExecPlan::Make(exec_context));
   // Drop projection since we only need to count rows
   const auto options = std::make_shared<ScanOptions>(*scan_options_);
   ARROW_ASSIGN_OR_RAISE(auto empty_projection,
                         ProjectionDescr::FromNames(std::vector<std::string>(),
-                                                   *scan_options_->dataset_schema));
+                                                   *scan_options_->dataset_schema,
+                                                   scan_options_->add_augmented_fields));
   SetProjection(options.get(), empty_projection);
 
   auto total = std::make_shared<std::atomic<int64_t>>(0);
@@ -737,16 +756,16 @@ Future<int64_t> AsyncScanner::CountRowsAsync(Executor* executor) {
             });
       });
 
-  compute::Declaration count_plan = compute::Declaration::Sequence(
+  acero::Declaration count_plan = acero::Declaration::Sequence(
       {{"scan",
         ScanNodeOptions{std::make_shared<FragmentDataset>(scan_options_->dataset_schema,
                                                           std::move(fragment_gen)),
                         options}},
-       {"project", compute::ProjectNodeOptions{{options->filter}, {"mask"}}},
-       {"aggregate", compute::AggregateNodeOptions{{compute::Aggregate{
+       {"project", acero::ProjectNodeOptions{{options->filter}, {"mask"}}},
+       {"aggregate", acero::AggregateNodeOptions{{compute::Aggregate{
                          "sum", nullptr, "mask", "selected_count"}}}}});
 
-  return compute::DeclarationToBatchesAsync(std::move(count_plan), exec_context)
+  return acero::DeclarationToBatchesAsync(std::move(count_plan), exec_context)
       .Then([total](const RecordBatchVector& batches) -> Result<int64_t> {
         DCHECK_EQ(1, batches.size());
         ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Scalar> count_scalar,
@@ -812,7 +831,8 @@ Result<ProjectionDescr> ProjectionDescr::FromExpressions(
 }
 
 Result<ProjectionDescr> ProjectionDescr::FromNames(std::vector<std::string> names,
-                                                   const Schema& dataset_schema) {
+                                                   const Schema& dataset_schema,
+                                                   bool add_augmented_fields) {
   std::vector<compute::Expression> exprs(names.size());
   for (size_t i = 0; i < exprs.size(); ++i) {
     // If name isn't in schema, try finding it by dotted path.
@@ -830,15 +850,19 @@ Result<ProjectionDescr> ProjectionDescr::FromNames(std::vector<std::string> name
     }
   }
   auto fields = dataset_schema.fields();
-  for (const auto& aug_field : kAugmentedFields) {
-    fields.push_back(aug_field);
+  if (add_augmented_fields) {
+    for (const auto& aug_field : kAugmentedFields) {
+      fields.push_back(aug_field);
+    }
   }
   return ProjectionDescr::FromExpressions(std::move(exprs), std::move(names),
                                           Schema(fields, dataset_schema.metadata()));
 }
 
-Result<ProjectionDescr> ProjectionDescr::Default(const Schema& dataset_schema) {
-  return ProjectionDescr::FromNames(dataset_schema.field_names(), dataset_schema);
+Result<ProjectionDescr> ProjectionDescr::Default(const Schema& dataset_schema,
+                                                 bool add_augmented_fields) {
+  return ProjectionDescr::FromNames(dataset_schema.field_names(), dataset_schema,
+                                    add_augmented_fields);
 }
 
 void SetProjection(ScanOptions* options, ProjectionDescr projection) {
@@ -883,7 +907,8 @@ const std::shared_ptr<Schema>& ScannerBuilder::projected_schema() const {
 Status ScannerBuilder::Project(std::vector<std::string> columns) {
   ARROW_ASSIGN_OR_RAISE(
       auto projection,
-      ProjectionDescr::FromNames(std::move(columns), *scan_options_->dataset_schema));
+      ProjectionDescr::FromNames(std::move(columns), *scan_options_->dataset_schema,
+                                 scan_options_->add_augmented_fields));
   SetProjection(scan_options_.get(), std::move(projection));
   return Status::OK();
 }
@@ -948,7 +973,7 @@ Status ScannerBuilder::FragmentScanOptions(
   return Status::OK();
 }
 
-Status ScannerBuilder::Backpressure(compute::BackpressureOptions backpressure) {
+Status ScannerBuilder::Backpressure(acero::BackpressureOptions backpressure) {
   scan_options_->backpressure = backpressure;
   return Status::OK();
 }
@@ -968,9 +993,9 @@ Result<std::shared_ptr<Scanner>> ScannerBuilder::Finish() {
 
 namespace {
 
-Result<compute::ExecNode*> MakeScanNode(compute::ExecPlan* plan,
-                                        std::vector<compute::ExecNode*> inputs,
-                                        const compute::ExecNodeOptions& options) {
+Result<acero::ExecNode*> MakeScanNode(acero::ExecPlan* plan,
+                                      std::vector<acero::ExecNode*> inputs,
+                                      const acero::ExecNodeOptions& options) {
   const auto& scan_node_options = checked_cast<const ScanNodeOptions&>(options);
   auto scan_options = scan_node_options.scan_options;
   auto dataset = scan_node_options.dataset;
@@ -1036,19 +1061,21 @@ Result<compute::ExecNode*> MakeScanNode(compute::ExecPlan* plan,
       });
 
   auto fields = scan_options->dataset_schema->fields();
-  for (const auto& aug_field : kAugmentedFields) {
-    fields.push_back(aug_field);
+  if (scan_options->add_augmented_fields) {
+    for (const auto& aug_field : kAugmentedFields) {
+      fields.push_back(aug_field);
+    }
   }
 
-  return compute::MakeExecNode(
+  return acero::MakeExecNode(
       "source", plan, {},
-      compute::SourceNodeOptions{schema(std::move(fields)), std::move(gen)});
+      acero::SourceNodeOptions{schema(std::move(fields)), std::move(gen)});
 }
 
-Result<compute::ExecNode*> MakeAugmentedProjectNode(
-    compute::ExecPlan* plan, std::vector<compute::ExecNode*> inputs,
-    const compute::ExecNodeOptions& options) {
-  const auto& project_options = checked_cast<const compute::ProjectNodeOptions&>(options);
+Result<acero::ExecNode*> MakeAugmentedProjectNode(acero::ExecPlan* plan,
+                                                  std::vector<acero::ExecNode*> inputs,
+                                                  const acero::ExecNodeOptions& options) {
+  const auto& project_options = checked_cast<const acero::ProjectNodeOptions&>(options);
   auto exprs = project_options.expressions;
   auto names = project_options.names;
 
@@ -1063,14 +1090,14 @@ Result<compute::ExecNode*> MakeAugmentedProjectNode(
     exprs.push_back(compute::field_ref(aug_field->name()));
     names.push_back(aug_field->name());
   }
-  return compute::MakeExecNode(
+  return acero::MakeExecNode(
       "project", plan, std::move(inputs),
-      compute::ProjectNodeOptions{std::move(exprs), std::move(names)});
+      acero::ProjectNodeOptions{std::move(exprs), std::move(names)});
 }
 
-Result<compute::ExecNode*> MakeOrderedSinkNode(compute::ExecPlan* plan,
-                                               std::vector<compute::ExecNode*> inputs,
-                                               const compute::ExecNodeOptions& options) {
+Result<acero::ExecNode*> MakeOrderedSinkNode(acero::ExecPlan* plan,
+                                             std::vector<acero::ExecNode*> inputs,
+                                             const acero::ExecNodeOptions& options) {
   if (inputs.size() != 1) {
     return Status::Invalid("Ordered SinkNode requires exactly 1 input, got ",
                            inputs.size());
@@ -1079,8 +1106,8 @@ Result<compute::ExecNode*> MakeOrderedSinkNode(compute::ExecPlan* plan,
 
   AsyncGenerator<std::optional<compute::ExecBatch>> unordered;
   ARROW_ASSIGN_OR_RAISE(auto node,
-                        compute::MakeExecNode("sink", plan, std::move(inputs),
-                                              compute::SinkNodeOptions{&unordered}));
+                        acero::MakeExecNode("sink", plan, std::move(inputs),
+                                            acero::SinkNodeOptions{&unordered}));
 
   const Schema& schema = *input->output_schema();
   ARROW_ASSIGN_OR_RAISE(FieldPath match, FieldRef("__fragment_index").FindOne(schema));
@@ -1140,7 +1167,7 @@ Result<compute::ExecNode*> MakeOrderedSinkNode(compute::ExecPlan* plan,
            last_in_fragment(*prev) && batch_index(*next) == 0;
   };
 
-  const auto& sink_options = checked_cast<const compute::SinkNodeOptions&>(options);
+  const auto& sink_options = checked_cast<const acero::SinkNodeOptions&>(options);
   *sink_options.generator =
       MakeSequencingGenerator(std::move(unordered), left_after_right, is_next,
                               std::make_optional(std::move(before_any)));
@@ -1151,7 +1178,7 @@ Result<compute::ExecNode*> MakeOrderedSinkNode(compute::ExecPlan* plan,
 }  // namespace
 
 namespace internal {
-void InitializeScanner(arrow::compute::ExecFactoryRegistry* registry) {
+void InitializeScanner(arrow::acero::ExecFactoryRegistry* registry) {
   DCHECK_OK(registry->AddFactory("scan", MakeScanNode));
   DCHECK_OK(registry->AddFactory("ordered_sink", MakeOrderedSinkNode));
   DCHECK_OK(registry->AddFactory("augmented_project", MakeAugmentedProjectNode));
