@@ -24,6 +24,7 @@
 #include <google/protobuf/any.pb.h>
 
 #include "arrow/buffer.h"
+#include "arrow/flight/serialization_internal.h"
 #include "arrow/flight/sql/protocol_internal.h"
 #include "arrow/flight/types.h"
 #include "arrow/io/memory.h"
@@ -40,24 +41,9 @@ namespace sql {
 namespace {
 arrow::Result<FlightDescriptor> GetFlightDescriptorForCommand(
     const google::protobuf::Message& command) {
-  google::protobuf::Any any;
-#if PROTOBUF_VERSION >= 3015000
-  if (!any.PackFrom(command)) {
-    return Status::SerializationError("Failed to pack ", command.GetTypeName());
-  }
-#else
-  any.PackFrom(command);
-#endif
-
-  std::string buf;
-#if PROTOBUF_VERSION >= 3015000
-  if (!any.SerializeToString(&buf)) {
-    return Status::SerializationError("Failed to serialize ", command.GetTypeName());
-  }
-#else
-  any.SerializeToString(&buf);
-#endif
-  return FlightDescriptor::Command(buf);
+  FlightDescriptor descriptor;
+  RETURN_NOT_OK(flight::internal::PackProtoCommand(command, &descriptor));
+  return descriptor;
 }
 
 arrow::Result<std::unique_ptr<FlightInfo>> GetFlightInfoForCommand(
@@ -76,32 +62,14 @@ arrow::Result<std::unique_ptr<SchemaResult>> GetSchemaForCommand(
   return client->GetSchema(options, descriptor);
 }
 
-::arrow::Result<Action> PackAction(const std::string& action_type,
-                                   const google::protobuf::Message& message) {
-  google::protobuf::Any any;
-#if PROTOBUF_VERSION >= 3015000
-  if (!any.PackFrom(message)) {
-    return Status::SerializationError("Could not pack ", message.GetTypeName(),
-                                      " into Any");
-  }
-#else
-  any.PackFrom(message);
-#endif
-
-  std::string buffer;
-#if PROTOBUF_VERSION >= 3015000
-  if (!any.SerializeToString(&buffer)) {
-    return Status::SerializationError("Could not serialize packed ",
-                                      message.GetTypeName());
-  }
-#else
-  any.SerializeToString(&buffer);
-#endif
-
-  Action action;
-  action.type = action_type;
-  action.body = Buffer::FromString(std::move(buffer));
-  return action;
+// Pack a protobuf action and send it to the server.
+arrow::Result<std::unique_ptr<ResultStream>> DoProtoAction(
+    FlightSqlClient* client, const FlightCallOptions& options, std::string action_type,
+    const google::protobuf::Message& action) {
+  Action packed_action;
+  RETURN_NOT_OK(
+      flight::internal::PackProtoAction(std::move(action_type), action, &packed_action));
+  return client->DoAction(options, packed_action);
 }
 
 void SetPlan(const SubstraitPlan& plan, flight_sql_pb::SubstraitPlan* pb_plan) {
@@ -256,6 +224,114 @@ arrow::Result<int64_t> FlightSqlClient::ExecuteSubstraitUpdate(
   return update_result.record_count();
 }
 
+arrow::Result<int64_t> FlightSqlClient::ExecuteIngest(
+    const FlightCallOptions& options, const std::shared_ptr<RecordBatchReader>& reader,
+    const TableDefinitionOptions& table_definition_options, const std::string& table,
+    const std::optional<std::string>& schema, const std::optional<std::string>& catalog,
+    const bool temporary, const Transaction& transaction,
+    const std::unordered_map<std::string, std::string>& ingest_options) {
+  flight_sql_pb::CommandStatementIngest command;
+
+  flight_sql_pb::CommandStatementIngest_TableDefinitionOptions*
+      pb_table_definition_options =
+          new flight_sql_pb::CommandStatementIngest_TableDefinitionOptions();
+  switch (table_definition_options.if_not_exist) {
+    case TableDefinitionOptionsTableNotExistOption::kUnspecified:
+      pb_table_definition_options->set_if_not_exist(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableNotExistOption_TABLE_NOT_EXIST_OPTION_UNSPECIFIED);  // NOLINT(whitespace/line_length)
+      break;
+    case TableDefinitionOptionsTableNotExistOption::kCreate:
+      pb_table_definition_options->set_if_not_exist(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableNotExistOption_TABLE_NOT_EXIST_OPTION_CREATE);  // NOLINT(whitespace/line_length)
+      break;
+    case TableDefinitionOptionsTableNotExistOption::kFail:
+      pb_table_definition_options->set_if_not_exist(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableNotExistOption_TABLE_NOT_EXIST_OPTION_FAIL);  // NOLINT(whitespace/line_length)
+      break;
+
+    default:
+      break;
+  }
+
+  switch (table_definition_options.if_exists) {
+    case TableDefinitionOptionsTableExistsOption::kUnspecified:
+      pb_table_definition_options->set_if_exists(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableExistsOption_TABLE_EXISTS_OPTION_UNSPECIFIED);  // NOLINT(whitespace/line_length)
+      break;
+    case TableDefinitionOptionsTableExistsOption::kFail:
+      pb_table_definition_options->set_if_exists(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableExistsOption_TABLE_EXISTS_OPTION_FAIL);  // NOLINT(whitespace/line_length)
+      break;
+    case TableDefinitionOptionsTableExistsOption::kAppend:
+      pb_table_definition_options->set_if_exists(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableExistsOption_TABLE_EXISTS_OPTION_APPEND);  // NOLINT(whitespace/line_length)
+      break;
+    case TableDefinitionOptionsTableExistsOption::kReplace:
+      pb_table_definition_options->set_if_exists(
+          flight_sql_pb::
+              CommandStatementIngest_TableDefinitionOptions_TableExistsOption_TABLE_EXISTS_OPTION_REPLACE);  // NOLINT(whitespace/line_length)
+      break;
+
+    default:
+      break;
+  }
+
+  command.set_allocated_table_definition_options(pb_table_definition_options);
+  command.set_table(table);
+
+  if (schema.has_value()) {
+    command.set_schema(schema.value());
+  }
+
+  if (catalog.has_value()) {
+    command.set_catalog(catalog.value());
+  }
+
+  command.set_temporary(temporary);
+
+  if (transaction.is_valid()) {
+    command.set_transaction_id(transaction.transaction_id());
+  }
+
+  auto command_options = command.mutable_options();
+  for (const auto& [key, val] : ingest_options) {
+    (*command_options)[key] = val;
+  }
+
+  ARROW_ASSIGN_OR_RAISE(FlightDescriptor descriptor,
+                        GetFlightDescriptorForCommand(command));
+
+  auto reader_ = reader.get();
+  ARROW_ASSIGN_OR_RAISE(auto stream, DoPut(options, descriptor, reader_->schema()));
+
+  while (true) {
+    ARROW_ASSIGN_OR_RAISE(auto batch, reader_->Next());
+    if (!batch) break;
+    ARROW_RETURN_NOT_OK(stream.writer->WriteRecordBatch(*batch));
+  }
+
+  ARROW_RETURN_NOT_OK(stream.writer->DoneWriting());
+  std::shared_ptr<Buffer> metadata;
+  ARROW_RETURN_NOT_OK(stream.reader->ReadMetadata(&metadata));
+  ARROW_RETURN_NOT_OK(stream.writer->Close());
+
+  if (!metadata) return Status::IOError("Server did not send a response");
+
+  flight_sql_pb::DoPutUpdateResult update_result;
+  if (!update_result.ParseFromArray(metadata->data(),
+                                    static_cast<int>(metadata->size()))) {
+    return Status::Invalid("Unable to parse DoPutUpdateResult");
+  }
+
+  return update_result.record_count();
+}
+
 arrow::Result<std::unique_ptr<FlightInfo>> FlightSqlClient::GetCatalogs(
     const FlightCallOptions& options) {
   flight_sql_pb::CommandGetCatalogs command;
@@ -486,9 +562,8 @@ arrow::Result<std::shared_ptr<PreparedStatement>> FlightSqlClient::Prepare(
     request.set_transaction_id(transaction.transaction_id());
   }
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("CreatePreparedStatement", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action))
-
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "CreatePreparedStatement", request))
   return PreparedStatement::ParseResponse(this, std::move(results));
 }
 
@@ -501,9 +576,8 @@ arrow::Result<std::shared_ptr<PreparedStatement>> FlightSqlClient::PrepareSubstr
     request.set_transaction_id(transaction.transaction_id());
   }
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("CreatePreparedSubstraitPlan", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action))
-
+  ARROW_ASSIGN_OR_RAISE(
+      auto results, DoProtoAction(this, options, "CreatePreparedSubstraitPlan", request));
   return PreparedStatement::ParseResponse(this, std::move(results));
 }
 
@@ -574,7 +648,7 @@ arrow::Result<std::unique_ptr<FlightInfo>> PreparedStatement::Execute(
                                          parameter_binding_.get()));
   }
   ARROW_ASSIGN_OR_RAISE(auto flight_info, client_->GetFlightInfo(options, descriptor));
-  return std::move(flight_info);
+  return flight_info;
 }
 
 arrow::Result<int64_t> PreparedStatement::ExecuteUpdate(
@@ -648,14 +722,12 @@ Status PreparedStatement::Close(const FlightCallOptions& options) {
   if (is_closed_) {
     return Status::Invalid("Statement with handle '", handle_, "' already closed");
   }
-
   flight_sql_pb::ActionClosePreparedStatementRequest request;
   request.set_prepared_statement_handle(handle_);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("ClosePreparedStatement", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, client_->DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(
+      auto results, DoProtoAction(client_, options, "ClosePreparedStatement", request));
   ARROW_RETURN_NOT_OK(results->Drain());
-
   is_closed_ = true;
   return Status::OK();
 }
@@ -664,8 +736,8 @@ Status PreparedStatement::Close(const FlightCallOptions& options) {
     const FlightCallOptions& options) {
   flight_sql_pb::ActionBeginTransactionRequest request;
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("BeginTransaction", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "BeginTransaction", request));
 
   flight_sql_pb::ActionBeginTransactionResult transaction;
   ARROW_RETURN_NOT_OK(ReadResult(results.get(), &transaction));
@@ -681,15 +753,14 @@ Status PreparedStatement::Close(const FlightCallOptions& options) {
     const FlightCallOptions& options, const Transaction& transaction,
     const std::string& name) {
   flight_sql_pb::ActionBeginSavepointRequest request;
-
   if (!transaction.is_valid()) {
     return Status::Invalid("Must provide an active transaction");
   }
   request.set_transaction_id(transaction.transaction_id());
   request.set_name(name);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("BeginSavepoint", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "BeginSavepoint", request));
 
   flight_sql_pb::ActionBeginSavepointResult savepoint;
   ARROW_RETURN_NOT_OK(ReadResult(results.get(), &savepoint));
@@ -704,41 +775,34 @@ Status PreparedStatement::Close(const FlightCallOptions& options) {
 Status FlightSqlClient::Commit(const FlightCallOptions& options,
                                const Transaction& transaction) {
   flight_sql_pb::ActionEndTransactionRequest request;
-
   if (!transaction.is_valid()) {
     return Status::Invalid("Must provide an active transaction");
   }
   request.set_transaction_id(transaction.transaction_id());
   request.set_action(flight_sql_pb::ActionEndTransactionRequest::END_TRANSACTION_COMMIT);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("EndTransaction", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
-
-  ARROW_RETURN_NOT_OK(results->Drain());
-  return Status::OK();
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "EndTransaction", request));
+  return results->Drain();
 }
 
 Status FlightSqlClient::Release(const FlightCallOptions& options,
                                 const Savepoint& savepoint) {
   flight_sql_pb::ActionEndSavepointRequest request;
-
   if (!savepoint.is_valid()) {
     return Status::Invalid("Must provide an active savepoint");
   }
   request.set_savepoint_id(savepoint.savepoint_id());
   request.set_action(flight_sql_pb::ActionEndSavepointRequest::END_SAVEPOINT_RELEASE);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("EndSavepoint", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
-
-  ARROW_RETURN_NOT_OK(results->Drain());
-  return Status::OK();
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "EndSavepoint", request));
+  return results->Drain();
 }
 
 Status FlightSqlClient::Rollback(const FlightCallOptions& options,
                                  const Transaction& transaction) {
   flight_sql_pb::ActionEndTransactionRequest request;
-
   if (!transaction.is_valid()) {
     return Status::Invalid("Must provide an active transaction");
   }
@@ -746,38 +810,33 @@ Status FlightSqlClient::Rollback(const FlightCallOptions& options,
   request.set_action(
       flight_sql_pb::ActionEndTransactionRequest::END_TRANSACTION_ROLLBACK);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("EndTransaction", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
-
-  ARROW_RETURN_NOT_OK(results->Drain());
-  return Status::OK();
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "EndTransaction", request));
+  return results->Drain();
 }
 
 Status FlightSqlClient::Rollback(const FlightCallOptions& options,
                                  const Savepoint& savepoint) {
   flight_sql_pb::ActionEndSavepointRequest request;
-
   if (!savepoint.is_valid()) {
     return Status::Invalid("Must provide an active savepoint");
   }
   request.set_savepoint_id(savepoint.savepoint_id());
   request.set_action(flight_sql_pb::ActionEndSavepointRequest::END_SAVEPOINT_ROLLBACK);
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("EndSavepoint", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action));
-
-  ARROW_RETURN_NOT_OK(results->Drain());
-  return Status::OK();
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "EndSavepoint", request));
+  return results->Drain();
 }
 
 ::arrow::Result<CancelResult> FlightSqlClient::CancelQuery(
     const FlightCallOptions& options, const FlightInfo& info) {
-  flight_sql_pb::ActionCancelQueryRequest request;
+  flight_sql_pb::ActionCancelQueryRequest cancel_query;
   ARROW_ASSIGN_OR_RAISE(auto serialized_info, info.SerializeToString());
-  request.set_info(std::move(serialized_info));
+  cancel_query.set_info(std::move(serialized_info));
 
-  ARROW_ASSIGN_OR_RAISE(auto action, PackAction("CancelQuery", request));
-  ARROW_ASSIGN_OR_RAISE(auto results, DoAction(options, action))
+  ARROW_ASSIGN_OR_RAISE(auto results,
+                        DoProtoAction(this, options, "CancelQuery", cancel_query));
 
   flight_sql_pb::ActionCancelQueryResult result;
   ARROW_RETURN_NOT_OK(ReadResult(results.get(), &result));
