@@ -33,11 +33,20 @@ Span::Span() noexcept {
   details = std::make_unique<::arrow::internal::tracing::SpanImpl>();
 }
 
+bool Span::valid() const {
+  return static_cast<::arrow::internal::tracing::SpanImpl*>(details.get())->valid();
+}
+
+void Span::reset() { details.reset(); }
+
 #else
 
 Span::Span() noexcept { /* details is left a nullptr */
 }
 
+bool Span::valid() const { return false; }
+void Span::reset() {}
+
 #endif
 
 }  // namespace tracing
