@@ -49,7 +49,7 @@ binary_paths <- readLines(tasks_yml) |>
 
 artifactory_root <- "https://apache.jfrog.io/artifactory/arrow/r/%s/libarrow/bin/%s"
 
-# Get the checksuym file from the artifactory
+# Get the checksum file from the artifactory
 for (path in binary_paths) {
   sha_path <- paste0(path, ".sha512")
   file <- file.path("tools/checksums", sha_path)
@@ -62,7 +62,7 @@ for (path in binary_paths) {
   if (grepl("windows", path)) {
     cat(paste0("Converting ", path, " to windows style line endings\n"))
     # UNIX style line endings cause errors with mysys2 sha512sum
-    sed_status <- system2("sed", args = c("-i", "s/\\\\r//", file))
+    sed_status <- system2("sed", args = c("-i", "-e", "s/\\\\r//", file))
     if (sed_status != 0) {
       stop("Failed to remove \\r from windows checksum file. Exit code: ", sed_status)
     }
